/* Copyright 2009 Jiayao Yu

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
package com.jiayao.reviewfinder;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.DialogInterface.OnClickListener;
import android.content.pm.ActivityInfo;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;

import com.google.zxing.integration.android.IntentIntegrator;
import com.google.zxing.integration.android.IntentResult;

public class Main extends Activity {

  private static final String TAG = "Main";
  private static final int SETTINGS_ID = Menu.FIRST;
  private static final int ABOUT_ID = Menu.FIRST + 1;
  protected static final int ACTION_SEARCH_BARCODE = 0;
  private static final int ACTION_SEARCH_TITLE = 1;
  private static final int ACTION_SET_SEARCH_TEXT = 2;
  private static final int ACTION_POPULATE_TITLE_LIST = 3;
  protected static final String DEFAULT_SEARCH_ENGINE = "default_search_engine";
  
  private ListView resultListView;
  private Map<String, String> results;
  private Handler serviceHandler;
  private Handler mainHandler;
  private SearchEngine searchEngine;
  private ProgressDialog mProgressDialog;

  /** Called when the activity is first created. */
  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.main);
    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
    resultListView = (ListView) findViewById(R.id.ResultList);
    resultListView.setOnItemClickListener(new OnItemClickListener() {
      public void onItemClick(AdapterView<?> parentView, View view, int pos, long id) {
	String title = ((TextView) view).getText().toString();
	browseTo(results.get(title));
      }
    });
    findViewById(R.id.SearchButton).setOnClickListener(new Button.OnClickListener() {
      public void onClick(View v) {
	SharedPreferences prefs = getPreferences(MODE_PRIVATE);
	String engineName = prefs.getString(DEFAULT_SEARCH_ENGINE, getString(R.string.site_metacritic));
	searchEngine = SearchEngineFactory.newGoogleSearchEngine(engineName);
	Message.obtain(serviceHandler,ACTION_SEARCH_TITLE, getSearchText()).sendToTarget();
	startProgressDialog();
      }
    });
    findViewById(R.id.ScanButton).setOnClickListener(new Button.OnClickListener() {
      public void onClick(View v) {
	startScanning();
      }
    });
    
    HandlerThread thread = new HandlerThread("ServiceStartArguments");
    thread.start();

    serviceHandler = new ServiceHandler(thread.getLooper());
    mainHandler = new MainHandler();
  }
  
  private void browseTo(String url) {
    Log.i(TAG, "BrowseTo " + url);
    startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(url)));
  }

  @Override
  public boolean onCreateOptionsMenu(Menu menu) {
    boolean result = super.onCreateOptionsMenu(menu);
    menu.add(0, SETTINGS_ID, 0, R.string.menu_settings);
    menu.add(0, ABOUT_ID, 0, R.string.menu_about);
    return result;
  }
  
  @Override
  public boolean onKeyDown(int keyCode, KeyEvent event) {
    switch (keyCode) {
      case KeyEvent.KEYCODE_CAMERA:
	startScanning();
	break;
      case KeyEvent.KEYCODE_BACK:
	new AlertDialog.Builder(this).setMessage(R.string.exit_confirmation)
	.setPositiveButton(R.string.alert_dialog_ok, new OnClickListener() {
	  public void onClick(DialogInterface dialog, int which) {
	    finish();
	  }
	})
	.setNegativeButton(R.string.alert_dialog_cancel, null)
	.show();
	return true;
    }
    return super.onKeyDown(keyCode, event);
  }

  @Override
  public boolean onOptionsItemSelected(MenuItem item) {
    switch (item.getItemId()) {
    case ABOUT_ID:
      AlertDialog.Builder builder = new AlertDialog.Builder(this);
      builder.setTitle(getString(R.string.title_about));
      builder.setMessage(getString(R.string.message_about));
      builder.setPositiveButton(R.string.alert_dialog_ok, null);
      builder.show();
      break;
    case SETTINGS_ID:
      LayoutInflater factory = LayoutInflater.from(this);
      final View settingsView = factory.inflate(R.layout.settings, null);
      builder = new AlertDialog.Builder(this);
      final SharedPreferences.Editor editor = getPreferences(MODE_PRIVATE).edit();
      final RadioGroup group = (RadioGroup) settingsView.findViewById(R.id.search_engine_choices);
      builder.setTitle(getString(R.string.title_settings))
      .setView(settingsView)
      .setPositiveButton(R.string.alert_dialog_ok, new OnClickListener() {
	public void onClick(DialogInterface dialog, int which) {
	  editor.putString(DEFAULT_SEARCH_ENGINE, ((RadioButton)settingsView.findViewById(group.getCheckedRadioButtonId())).getText().toString());
	  editor.commit();
	}
      })
      .setNegativeButton(R.string.alert_dialog_cancel, null)
      .show();
      
      break;
    }
    return super.onOptionsItemSelected(item);
  }

  private void startScanning() {
    IntentIntegrator.initiateScan(this);
  }

  public void onActivityResult(int requestCode, int resultCode, Intent intent) {
    IntentResult scanResult = IntentIntegrator.parseActivityResult(requestCode, resultCode, intent);
    Barcode barcode = new Barcode(scanResult.getFormatName(), scanResult.getContents());
    if (barcode.getContent() == null) {
      return;
    }
    try {
      searchEngine = SearchEngineFactory.searchEngineForBarcode(barcode);
      if (scanResult != null) {
	Message.obtain(serviceHandler, ACTION_SEARCH_BARCODE, scanResult.getContents()).sendToTarget();
	startProgressDialog();
      }
    } catch (UnsupportedBarcodeException e) {
      alert("Unsupported barcode:" + barcode.getContent() + " " + barcode.getFormat());
      return;
    }
  }

  private void startProgressDialog() {
    mProgressDialog = new ProgressDialog(this);
    mProgressDialog.setTitle(R.string.searching);
    mProgressDialog.show();
  }

  public void searchBarcode(String barcode) {
    try {
      Map<String, String> results = searchEngine.search(barcode);
      if (results.size() > 0) {
	String title = results.keySet().iterator().next();
	String link = results.get(title);
	if (searchEngine.getRefiningSearchEngine() == null) {
	  mProgressDialog.dismiss();
	  // we have a definitive match
	  browseTo(link);
	} else {
	  mProgressDialog.setProgress(50);
	  Message.obtain(mainHandler, ACTION_SET_SEARCH_TEXT, title).sendToTarget();
	  searchEngine = searchEngine.getRefiningSearchEngine();
	  searchTitle(title);
	}
      } else {
	mProgressDialog.dismiss();
	alert(getString(R.string.no_results));
      }
    } catch (MalformedURLException e) {
      mProgressDialog.dismiss();
      e.printStackTrace();
    } catch (IOException e) {
      mProgressDialog.dismiss();
      alert("Unable to access the network.");
    } 
    
  }
  
  private void searchTitle(String query) {
    try {
      results = searchEngine.search(query);
      mProgressDialog.dismiss();
      Log.d(TAG, results.toString());
      if (results.size() == 0) {
        alert(getString(R.string.no_results));
      } else {
	Message.obtain(mainHandler, ACTION_POPULATE_TITLE_LIST, new ArrayList<String>(results.keySet())).sendToTarget();
      }
    } catch (IOException e) {
      mProgressDialog.dismiss();
      alert("Unable to access the network.");
    }
  }

  private String getSearchText() {
    return ((EditText) findViewById(R.id.SearchEditText)).getText().toString();
  }
  
  private void setSearchText(String text) {
    ((EditText) findViewById(R.id.SearchEditText)).setText(text);
  }

  public Map<String, String> getResults() {
    return results;
  }

  private void populateTitleList(List<String> titles) {
    resultListView.setAdapter(
	new ArrayAdapter<String>(this, R.layout.list_item, titles));
  }

  private void alert(String msg) {
    new AlertDialog.Builder(this).setMessage(msg).setPositiveButton(R.string.alert_dialog_ok, null)
	.show();
  }
  
  private final class ServiceHandler extends Handler {

    public ServiceHandler(Looper looper) {
	    super(looper);
    }

    @Override
    public void handleMessage(Message msg)
    {
      switch (msg.what) {
      case ACTION_SEARCH_BARCODE: 
  	searchBarcode((String) msg.obj);
  	break;
      case ACTION_SEARCH_TITLE:
	searchTitle((String) msg.obj);
	break;
      default:
        alert("Unrecognized message:" + msg.what);
      }
    }
  }
  
  private final class MainHandler extends Handler {
    private static final int ACTION_SAVE_RESULTS = 0;

    @Override
    public void handleMessage(Message msg)
    {
      switch (msg.what) {
      case ACTION_POPULATE_TITLE_LIST:
	populateTitleList((List<String>) msg.obj);
	break;
      case ACTION_SET_SEARCH_TEXT:
	setSearchText((String) msg.obj);
	break;
      }
    }
  }

}